unit elasp;
{ engine of Logic Analyzer }

{

 LAB byte structure:

     MSb7   6   5   4   3   2   1   0LSb
      CH4  CH3 CH2 CH1    OUT3 OUT2  OUT1


}


interface

const
   _ts_CH1=$10;
   _ts_CH2=$20;
   _ts_CH3=$40;
   _ts_CH4=$80;
   _CH1=$10;
   _CH2=$20;
   _CH3=$40;
   _CH4=$80;
   maxBuf=$6FF90;

type
   TLAB=array [0..$FFF0] of byte;
   TTrigSource= byte;
   TTrigEdge = (_te_falling,_te_rising);
var
   LAB1:^TLAB;
   LAB2:^TLAB;
   LAB3:^TLAB;
   LAB4:^TLAB;
   LAB5:^TLAB;
   LAB6:^TLAB;
   LAB7:^TLAB;
   LAB8:^TLAB;

   SPBA:word; { serial port basic address }
   TrigSource: TTrigSource;
   TrigEdge: TTrigEdge;
   SD: longint;
   TSD: double;
   Generator:byte;
   eerr: byte;
   OldTrig,CurTrig,chs:byte;
   NOT_CH:boolean;



{ basic functions }
function GetLabMem:boolean;
procedure FreeLabMem;
function Trigger:boolean;
procedure GetPutSamp(var ptr:longint);
procedure WrLabSamp(var ptr:longint;var samp:byte);
function RdLabSamp(var ptr:longint):byte;
procedure ClearBuf;
function Go:boolean;
function logVal(ptr:longint;ch:byte):byte;

implementation

uses
  crt,dos;
var
  ub1,ub2,ub3:byte; { universal byte }


function GetLabMem:boolean;
begin
  GetLabMem:= false;
  if MemAvail< 470000 then begin
    writeln('Out of memory.');
    exit;
  end;
  New(LAB1);
  New(LAB2);
  New(LAB3);
  New(LAB4);
  New(LAB5);
  New(LAB6);
  New(LAB7);
{  New(LAB8);}
  GetLabMem:= true;
end;

procedure FreeLabMem;
begin
  Dispose(LAB1);
  Dispose(LAB2);
  Dispose(LAB3);
  Dispose(LAB4);
  Dispose(LAB5);
  Dispose(LAB6);
  Dispose(LAB7);
{  Dispose(LAB8);}
end;


function RdLabSamp(var ptr:longint):byte;
{ cte sampl z bufferu }
begin
  eerr:=0;
  { LAB 1 }
  if ptr <= $FFF0 then begin
    RdLabSamp:= LAB1^[word(ptr)];
    exit;
  end;
  { LAB 2 }
  if ptr <= $1FFE0 then begin
    RdLabSamp:= LAB2^[word(ptr-$FFF0)];
    exit;
  end;
  { LAB 3 }
  if ptr <= $2FFD0 then begin
    RdLabSamp:= LAB3^[word(ptr-$1FFE0)];
    exit;
  end;
  { LAB 4 }
  if ptr <= $3FFC0 then begin
    RdLabSamp:= LAB4^[word(ptr-$2FFD0)];
    exit;
  end;
  { LAB 5 }
  if ptr <= $4FFB0 then begin
    RdLabSamp:= LAB5^[word(ptr-$3FFC0)];
    exit;
  end;
  { LAB 6 }
  if ptr <= $5FFA0 then begin
    RdLabSamp:= LAB6^[word(ptr-$4FFB0)];
    exit;
  end;
  { LAB 7 }
  if ptr <= $6FF90 then begin
    RdLabSamp:= LAB7^[word(ptr-$5FFA0)];
    exit;
  end;
(*  { LAB 8 }
  if ptr <= $7FF80 then begin
    RdLabSamp:= LAB8^[word(ptr-$6FF90)];
    exit;
  end;*)
  if ptr>$7FF80 then eerr:=$10;
end;


procedure WrLabSamp(var ptr:longint;var samp:byte);
{ umistuje sampl do bufferu }
begin
  eerr:=0;
  { LAB 1 }
  if ptr <= $FFF0 then begin
    LAB1^[word(ptr)]:= samp;
    exit;
  end;
  { LAB 2 }
  if ptr <= $1FFE0 then begin
    LAB2^[word(ptr-$FFF0)]:= samp;
    exit;
  end;
  { LAB 3 }
  if ptr <= $2FFD0 then begin
    LAB3^[word(ptr-$1FFE0)]:= samp;
    exit;
  end;
  { LAB 4 }
  if ptr <= $3FFC0 then begin
    LAB4^[word(ptr-$2FFD0)]:= samp;
    exit;
  end;
  { LAB 5 }
  if ptr <= $4FFB0 then begin
    LAB5^[word(ptr-$3FFC0)]:= samp;
    exit;
  end;
  { LAB 6 }
  if ptr <= $5FFA0 then begin
    LAB6^[word(ptr-$4FFB0)]:= samp;
    exit;
  end;
  { LAB 7 }
  if ptr <= $6FF90 then begin
    LAB7^[word(ptr-$5FFA0)]:= samp;
    exit;
  end;
(*  { LAB 8 }
  if ptr <= $7FF80 then begin
    LAB8^[word(ptr-$6FF90)]:= samp;
    exit;
  end;*)
  if ptr>$7FF80 then eerr:=$10;
end;

procedure GetPutSamp(var ptr:longint);
begin
  eerr:=0;
  ub1:= port[SPBA+6] and $F0;
    ub2:= RdLabSamp(ptr);
    port[SPBA+4]:= ub2 and $03;
    ub1:=ub1 or (ub2 and $07);
    WrLabSamp(ptr,ub1);
    ub2:= (ub2 and $04)*$10;
    port[SPBA+3]:= ub2;
end;

function Trigger:boolean;
var
  i:longint;
begin
  Trigger:= false;
  chs:=port[SPBA+6];
  curTrig:= byte((chs and TrigSource)=TrigSource);

  case TrigEdge of
    { ~~~\___ - falling edge detection }
    _te_falling: begin
      if (OldTrig=1) and (CurTrig=0) then begin
        Trigger:=true;
        exit;
      end;
    end;
    { ____/~~~~ - rising edge detection }
    _te_rising: begin
       if (OldTrig=0) and (CurTrig=1) then begin
        Trigger:=true;
        exit;
       end;
    end;
  end;

  OldTrig:= CurTrig;
end;

procedure ClearBuf;
var
  i:longint;
begin
  for i:=0 to $7FF80 do begin
    ub1:= RdLabSamp(i) or $0F;
    WrLabSamp(i,ub1);
  end;
end;

function Go:boolean;
var
  bufp,stime,ftime:longint;
  i:word;
  dt,frt,srt:double;
  sH,sM,sS,s100S,fH,fM,fS,f100S:word;
begin
  { sample delay }
  go:=true;
  { get start time }
  getTime(sH,sM,sS,s100S);
  stime:=(longint(sH)*360000);
  stime:=stime + (longint(sM)*6000)+(longint(sS)*100)+longint(s100S); { desitky ms }

  if generator=1 then begin
   { SLOW GENERATOR MODE }
   for bufp:=0 to $6FF90 {_$7FF80} do begin
    for i:=0 to SD do ub1:=port[SPBA+6];
    GetPutSamp(bufp);
    if (bufp mod 100) =0 then
      if keypressed then begin
        go:=false;
        break;
      end;
   end;
  end
  else begin
    { ULTRA FAST }
    if SD=0 then begin
      for bufp:=0 to $6FF90 do begin
        ub1:= port[SPBA+6] and $F0;
        WrLabSamp(bufp,ub1);
      end;
    end
    else begin
      { FAST }
      for bufp:=0 to $6FF90 do begin
        for i:=0 to SD do ub1:=port[SPBA+6];
        ub1:= port[SPBA+6] and $F0;
        WrLabSamp(bufp,ub1);
      end;
    end;
  end;

  { get end time }
  getTime(fH,fM,fS,f100S);
  ftime:=(longint(fH)*360000);
  ftime:= ftime +(longint(fM)*6000)+(longint(fS)*100)+longint(f100S); { desitky ms }
  frt:= ftime;
  srt:= stime;
  dt:= abs(frt-srt);
  TSD:= (dt*10000)/(bufp+1);
end;

function logVal(ptr:longint;ch:byte):byte;
begin
  ub1:= RdLabSamp(ptr);
  if not NOT_CH then
    logVal:= byte((ub1 and ch) =ch)
  else
    logVal:= byte(not ((ub1 and ch) =ch))
end;


begin

  SPBA:= $2F8;
end.